@@ -14,6 +14,11 @@ from esi_utils_rupture.quad_rupture import QuadRupture
 from esi_utils_rupture.point_rupture import PointRupture
 from esi_utils_rupture.distance import Distance
 from esi_shakelib.sites import Sites
+from esi_shakelib.utils.contexts import (
+    SitesContext,
+    DistancesContext,
+    RuptureContext,
+)
 
 
 class AutoName(Enum):
@@ -60,8 +65,8 @@ class Mechanism(AutoName):
 
 class FFSimmer:
     """
-    Class to perform monte carlo simulations of finite faults to derive mean ground
-    motions
+    Class to perform monte carlo simulations of finite faults to derive mean
+    ground motions
     """
 
     def __init__(self, rff, measure="median"):
@@ -70,7 +75,8 @@ class FFSimmer:
             rff (RandomFiniteFault object):
                 An instance of the RandomFiniteFault object.
             measure (str):
-                Either "mean" or "median" -- the measure of central tendency to report.
+                Either "mean" or "median" -- the measure of central tendency
+                to report.
         """
         if measure not in ("mean", "median"):
             raise ValueError(f"Unknown measure of central tendency: {measure}")
@@ -82,11 +88,11 @@ class FFSimmer:
         """
         Args:
             grid (Grid2D object):
-                Basically the vs30grid that we're mapping, used for its dimensions.
-                The site corrections have to be done elsewhere.
+                Basically the vs30grid that we're mapping, used for its
+                dimensions.  The site corrections have to be done elsewhere.
             gmpes (dictionary):
-                A dictionary of MultiGMPEs to be used for the simulations keyed to
-                myimts.
+                A dictionary of MultiGMPEs to be used for the simulations
+                keyed to myimts.
             myimts (list):
                 List of OQ IMTs to generate grids and transects for
 
@@ -94,10 +100,12 @@ class FFSimmer:
             dictionary:
                 {
                     oqimt: {
-                        "mean": 2D Array (the median or mean ground motion values, at
-                            a Vs30 of 760 m/s),
-                        "transect": Array (a transect of the grid at the repis below),
-                        "repi": Array (the Repis corresponding to "transect" above),
+                        "mean": 2D Array (the median or mean ground motion
+                            values, at a Vs30 of 760 m/s),
+                        "transect": Array (a transect of the grid at the repis
+                            below),
+                        "repi": Array (the Repis corresponding to "transect"
+                            above),
                         oqconst.StdDev.TOTAL: Array,
                         (if allowed:)
                         oqconst.StdDev.INTER_EVENT: Array,
@@ -164,11 +172,12 @@ class FFSimmer:
 
         nsim = self.rff.get_nsim()
         nimt = len(myimts)
-        amps = np.empty((nimt, nsim, nx * ny))
-        sigs = np.empty((nimt, nsim, nx * ny))
+        nporig = nx * ny
+        amps = np.zeros((nimt, nsim, nporig))
+        sigs = np.zeros((nimt, nsim, nporig))
         if len(sd_types) > 1:
-            taus = np.empty((nimt, nsim, nx * ny))
-            phis = np.empty((nimt, nsim, nx * ny))
+            taus = np.zeros((nimt, nsim, nporig))
+            phis = np.zeros((nimt, nsim, nporig))
 
         for idx in range(nsim):
             rupt = self.rff.retrieve_rupt(idx)
@@ -180,17 +189,38 @@ class FFSimmer:
                 if not isinstance(dctx.__dict__[k], np.ndarray):
                     continue
                 dctx.__dict__[k] = dctx.__dict__[k].flatten()
-            rctx = rupt.getRuptureContext(gmpes[myimts[0]])
+            rctx = rupt.getRuptureContext(gmpes[myimts[0]], (nporig,))
+            if idx == 0:
+                cols = []
+                arrs = []
+                for ctx in (sctx, dctx, rctx):
+                    for name, val in vars(ctx).items():
+                        if ctx == sctx and name in ["lons", "lats"]:
+                            continue
+                        if isinstance(val, np.ndarray):
+                            cols.append(name)
+                            arrs.append(
+                                np.zeros(nsim * np.size(val), dtype=val.dtype)
+                            )
+                cx = np.rec.fromarrays(arrs, names=cols)
+            for ctx in (sctx, dctx, rctx):
+                for name, val in vars(ctx).items():
+                    if isinstance(val, np.ndarray):
+                        arr = getattr(cx, name)
+                        istart = idx * np.size(val)
+                        arr[istart : istart + np.size(val)] = val[:]
 
-            for iimt, simt in enumerate(myimts):
-                mean, stddevs = gmpes[simt].get_mean_and_stddevs(
-                    sctx, rctx, dctx, simt, sd_types
-                )
-                amps[iimt, idx, :] = mean[0]
-                sigs[iimt, idx, :] = stddevs[0]
+        for iimt, simt in enumerate(myimts):
+            mean, stddevs = gmpes[simt].get_mean_and_stddevs(
+                cx, cx, cx, simt, sd_types
+            )
+            for idx in range(nsim):
+                istart = idx * nporig
+                amps[iimt, idx, :] = mean[0][istart : istart + nporig]
+                sigs[iimt, idx, :] = stddevs[0][istart : istart + nporig]
                 if len(sd_types) > 1:
-                    taus[iimt, idx, :] = stddevs[1]
-                    phis[iimt, idx, :] = stddevs[2]
+                    taus[iimt, idx, :] = stddevs[1][istart : istart + nporig]
+                    phis[iimt, idx, :] = stddevs[2][istart : istart + nporig]
 
         slat = np.linspace(sgd.ymin, sgd.ymax, sgd.ny)
         repi = prup.computeRepi(
@@ -219,9 +249,9 @@ class FFSimmer:
             try:
                 avg_extra_tau = np.var(np.mean(residuals, axis=1))
                 avg_extra_phi = np.mean(np.var(residuals, axis=1))
-            except RuntimeWarning as exc:
+            except RuntimeWarning:
                 # Don't have an answer, so just guess
-                print(f"Error in FFSimmer: {exc}")
+                print("Warning in FFSimmer: can't compute MER stats")
                 print("Using defaults")
                 avg_extra_tau = 0.20
                 avg_extra_phi = 0.80
@@ -244,12 +274,18 @@ class FFSimmer:
             # Take the mean over the realizations
             mean_sigs = np.sqrt(np.mean(sigs[iimt] ** 2, axis=0) + extra_vars)
             if len(sd_types) > 1:
-                mean_taus = np.sqrt(np.mean(taus[iimt] ** 2, axis=0) + extra_taus)
-                mean_phis = np.sqrt(np.mean(phis[iimt] ** 2, axis=0) + extra_phis)
+                mean_taus = np.sqrt(
+                    np.mean(taus[iimt] ** 2, axis=0) + extra_taus
+                )
+                mean_phis = np.sqrt(
+                    np.mean(phis[iimt] ** 2, axis=0) + extra_phis
+                )
 
             griddict[myimts[iimt]] = {
                 "mean": np.interp(grepi, repi, np.flip(mean_amps)),
-                oqconst.StdDev.TOTAL: np.interp(grepi, repi, np.flip(mean_sigs)),
+                oqconst.StdDev.TOTAL: np.interp(
+                    grepi, repi, np.flip(mean_sigs)
+                ),
                 "transect": np.flip(mean_amps),
                 "repi": repi,
             }
@@ -263,17 +299,19 @@ class FFSimmer:
 
         return griddict
 
-    def compute_points(self, sctx, gmpes, myimts, rock_vs30=760.0, soil_vs30=180.0):
+    def compute_points(
+        self, sctx, gmpes, myimts, rock_vs30=760.0, soil_vs30=180.0
+    ):
         """
-        Compute median (or mean) ground motions for a set of sites defined in "sctx"
-        for the imts in the list "myimts".
+        Compute median (or mean) ground motions for a set of sites defined in
+        "sctx" for the imts in the list "myimts".
 
         Args:
             sctx: (Sites object)
                 A sites context object set up for the points being output
             gmpes: (dictionary)
-                A dictionary (keyed to myimts) of gmpes to perform the ground-motion
-                  realizations.
+                A dictionary (keyed to myimts) of gmpes to perform the
+                ground-motion realizations.
             myimts (list)
                 List of OQ IMTs to generate rock and site amplified values for
 
@@ -318,9 +356,13 @@ class FFSimmer:
             if k == "_slots_":
                 continue
             sx.__dict__[k] = np.hstack(
-                (sctx.__dict__[k], sctx_rock.__dict__[k], sctx_soil.__dict__[k])
+                (
+                    sctx.__dict__[k],
+                    sctx_rock.__dict__[k],
+                    sctx_soil.__dict__[k],
+                )
             )
-        sx.sids = np.array(list(range(np.size(sx.lons))))
+        # sx.sids = np.array(list(range(np.size(sx.lons))))
         depths = np.zeros_like(sx.lons)
 
         nsd = len(sd_types)
@@ -331,41 +373,71 @@ class FFSimmer:
         nimt = len(myimts)
         nsim = self.rff.get_nsim()
 
-        amps = np.empty((nctx, nimt, nsim, nporig))
-        sigs = np.empty((nctx, nimt, nsim, nporig))
+        amps = np.zeros((nctx, nimt, nsim, nporig))
+        sigs = np.zeros((nctx, nimt, nsim, nporig))
         if nsd > 1:
-            taus = np.empty((nctx, nimt, nsim, nporig))
-            phis = np.empty((nctx, nimt, nsim, nporig))
+            taus = np.zeros((nctx, nimt, nsim, nporig))
+            phis = np.zeros((nctx, nimt, nsim, nporig))
 
+        llons = sx.lons.flatten()
+        llats = sx.lats.flatten()
+        ddepths = depths.flatten()
         for idx in range(nsim):
             rupt = self.rff.retrieve_rupt(idx)
             dist_obj = Distance(
                 gmpes[myimts[0]],
-                sx.lons.flatten(),
-                sx.lats.flatten(),
-                depths.flatten(),
+                llons,
+                llats,
+                ddepths,
                 rupt,
             )
             dctx = dist_obj.getDistanceContext()
-            rctx = rupt.getRuptureContext(gmpes[myimts[0]])
+            rctx = rupt.getRuptureContext(gmpes[myimts[0]], shape=llons.shape)
+            if idx == 0:
+                cols = []
+                arrs = []
+                for ctx in (sx, dctx, rctx):
+                    for name, val in vars(ctx).items():
+                        if ctx == sx and name in ["lons", "lats"]:
+                            continue
+                        if isinstance(val, np.ndarray):
+                            cols.append(name)
+                            arrs.append(
+                                np.zeros(nsim * np.size(val), dtype=val.dtype)
+                            )
+                cx = np.rec.fromarrays(arrs, names=cols)
+            for ctx in (sx, dctx, rctx):
+                for name, val in vars(ctx).items():
+                    if ctx == sx and name in ["lons", "lats"]:
+                        continue
+                    if isinstance(val, np.ndarray):
+                        arr = getattr(cx, name)
+                        istart = idx * np.size(val)
+                        arr[istart : istart + np.size(val)] = val[:]
 
-            for ix, oqimt in enumerate(myimts):
-                mean, stddevs = gmpes[oqimt].get_mean_and_stddevs(
-                    sx, rctx, dctx, [oqimt], sd_types
-                )
+        for ix, oqimt in enumerate(myimts):
+            mean, stddevs = gmpes[oqimt].get_mean_and_stddevs(
+                cx, cx, cx, [oqimt], sd_types
+            )
+            for idx in range(nsim):
+                istart = idx * nctx * nporig
                 for ictx in range(nctx):
                     amps[ictx, ix, idx, :] = mean[0][
-                        ictx * nporig : (ictx + 1) * nporig
+                        istart + ictx * nporig : istart + (ictx + 1) * nporig
                     ]
                     sigs[ictx, ix, idx, :] = stddevs[0][
-                        ictx * nporig : (ictx + 1) * nporig
+                        istart + ictx * nporig : istart + (ictx + 1) * nporig
                     ]
                     if nsd > 1:
                         taus[ictx, ix, idx, :] = stddevs[1][
-                            ictx * nporig : (ictx + 1) * nporig
+                            istart
+                            + ictx * nporig : istart
+                            + (ictx + 1) * nporig
                         ]
                         phis[ictx, ix, idx, :] = stddevs[2][
-                            ictx * nporig : (ictx + 1) * nporig
+                            istart
+                            + ictx * nporig : istart
+                            + (ictx + 1) * nporig
                         ]
 
         # Compute Mean, Sigma, Phi, and Tau for each IMT
@@ -379,16 +451,16 @@ class FFSimmer:
                     mean_amps = np.mean(amps[ictx, ix], axis=0)
                 # Compute the residuals for the simulations
                 residuals = amps[ictx, ix] - mean_amps
-                # Find the variance over the realizations, this is the variance we
-                # have to account for
+                # Find the variance over the realizations, this is the
+                # variance we have to account for
                 extra_vars = np.var(amps[ictx, ix], axis=0)
 
                 try:
                     avg_extra_tau = np.var(np.mean(residuals, axis=1))
                     avg_extra_phi = np.mean(np.var(residuals, axis=1))
-                except RuntimeWarning as exc:
+                except RuntimeWarning:
                     # Don't have an answer, so just guess
-                    print(f"Error in FFSimmer: {exc}")
+                    print("Warning in FFSimmer: can't compute MER stats")
                     print("Using defaults")
                     avg_extra_tau = 0.20
                     avg_extra_phi = 0.80
@@ -408,7 +480,9 @@ class FFSimmer:
                 extra_taus = avg_tau_wgt * extra_vars
                 extra_phis = avg_phi_wgt * extra_vars
 
-                mean_sigs = np.sqrt(np.mean(sigs[ictx, ix] ** 2, axis=0) + extra_vars)
+                mean_sigs = np.sqrt(
+                    np.mean(sigs[ictx, ix] ** 2, axis=0) + extra_vars
+                )
                 if len(sd_types) > 1:
                     mean_taus = np.sqrt(
                         np.mean(taus[ictx, ix] ** 2, axis=0) + extra_taus
@@ -421,7 +495,9 @@ class FFSimmer:
                     {
                         ctx_name: {
                             "mean": mean_amps.reshape(sctx.lons.shape),
-                            oqconst.StdDev.TOTAL: mean_sigs.reshape(sctx.lons.shape),
+                            oqconst.StdDev.TOTAL: mean_sigs.reshape(
+                                sctx.lons.shape
+                            ),
                         }
                     }
                 )
@@ -441,8 +517,8 @@ class FFSimmer:
             grid (Grid2D object):
                 The vs30grid that we're mapping.
             gmpes (dictionary):
-                A dictionary of MultiGMPEs to be used for the simulations keyed to
-                myimts.
+                A dictionary of MultiGMPEs to be used for the simulations
+                keyed to myimts.
             myimts (list):
                 List of OQ IMTs to generate grids and transects for
 
@@ -450,8 +526,8 @@ class FFSimmer:
             dictionary:
                 {
                     oqimt: {
-                        "mean": 2D Array (the median or mean ground motion values, at
-                            a Vs30 of 760 m/s),
+                        "mean": 2D Array (the median or mean ground motion
+                            values, at a Vs30 of 760 m/s),
                         oqconst.StdDev.TOTAL: Array,
                         (if allowed:)
                         oqconst.StdDev.INTER_EVENT: Array,
@@ -480,11 +556,12 @@ class FFSimmer:
 
         nsim = self.rff.get_nsim()
         nimt = len(myimts)
-        amps = np.empty((nimt, nsim, nx * ny))
-        sigs = np.empty((nimt, nsim, nx * ny))
+        nporig = nx * ny
+        amps = np.zeros((nimt, nsim, nporig))
+        sigs = np.zeros((nimt, nsim, nporig))
         if len(sd_types) > 1:
-            taus = np.empty((nimt, nsim, nx * ny))
-            phis = np.empty((nimt, nsim, nx * ny))
+            taus = np.zeros((nimt, nsim, nporig))
+            phis = np.zeros((nimt, nsim, nporig))
 
         for idx in range(nsim):
             rupt = self.rff.retrieve_rupt(idx)
@@ -496,7 +573,7 @@ class FFSimmer:
                 if not isinstance(dctx.__dict__[k], np.ndarray):
                     continue
                 dctx.__dict__[k] = dctx.__dict__[k].flatten()
-            rctx = rupt.getRuptureContext(gmpes[myimts[0]])
+            rctx = rupt.getRuptureContext(gmpes[myimts[0]], (nporig,))
 
             for iimt, simt in enumerate(myimts):
                 mean, stddevs = gmpes[simt].get_mean_and_stddevs(
@@ -520,9 +597,9 @@ class FFSimmer:
             try:
                 avg_extra_tau = np.var(np.mean(residuals, axis=1))
                 avg_extra_phi = np.mean(np.var(residuals, axis=1))
-            except RuntimeWarning as exc:
+            except RuntimeWarning:
                 # Don't have an answer, so just guess
-                print(f"Error in FFSimmer: {exc}")
+                print("Warning in FFSimmer: can't compute MER stats")
                 print("Using defaults")
                 avg_extra_tau = 0.20
                 avg_extra_phi = 0.80
@@ -545,19 +622,23 @@ class FFSimmer:
             # Take the mean over the realizations
             mean_sigs = np.sqrt(np.mean(sigs[iimt] ** 2, axis=0) + extra_vars)
             if len(sd_types) > 1:
-                mean_taus = np.sqrt(np.mean(taus[iimt] ** 2, axis=0) + extra_taus)
-                mean_phis = np.sqrt(np.mean(phis[iimt] ** 2, axis=0) + extra_phis)
+                mean_taus = np.sqrt(
+                    np.mean(taus[iimt] ** 2, axis=0) + extra_taus
+                )
+                mean_phis = np.sqrt(
+                    np.mean(phis[iimt] ** 2, axis=0) + extra_phis
+                )
 
             griddict[myimts[iimt]] = {
                 "mean": mean_amps.reshape((ny, nx)),
                 oqconst.StdDev.TOTAL: mean_sigs.reshape((ny, nx)),
             }
             if len(sd_types) > 1:
-                griddict[myimts[iimt]][oqconst.StdDev.INTER_EVENT] = mean_taus.reshape(
-                    (ny, nx)
+                griddict[myimts[iimt]][oqconst.StdDev.INTER_EVENT] = (
+                    mean_taus.reshape((ny, nx))
                 )
-                griddict[myimts[iimt]][oqconst.StdDev.INTRA_EVENT] = mean_phis.reshape(
-                    (ny, nx)
+                griddict[myimts[iimt]][oqconst.StdDev.INTRA_EVENT] = (
+                    mean_phis.reshape((ny, nx))
                 )
 
         return griddict
@@ -585,25 +666,29 @@ class RandomFiniteFault(object):
         """
         Args:
             origin (Origin object): The origin of the earthquake. Can include
-                attributes "tectonic_region" ("Active", "Stable", "Subduction" are
-                recognized) and "mech" ("ALL", "NM" (normal), "RS" (reverse), "SS"
-                (strike-slip)).
+                attributes "tectonic_region" ("Active", "Stable", "Subduction"
+                are recognized) and "mech" ("ALL", "NM" (normal), "RS"
+                (reverse), "SS" (strike-slip)).
             nsim (int): The number of simulations to run.
-            min_strike/max_strike (float/float): Limit the range of the strike (in
-                degrees) of realized faults. The default -180/180 allows any strike.
-            min_dip/max_dip (float/float): Limit the range of the dip (in degrees)
-                of the realized faults. The default is to use the "mech" of the
-                origin to limit the range or, if none, use the "ALL" mech which
-                defaults to 35/90.
-            dx_min_frac/dx_max_frac (float/float): Limit the range of the along-
-                strike placement of the hypocenter to from dx_min_frac * length to
-                dx_max_frac * length. (default 0.0/1.0)
-            delta_depth (float): The depth range of the hypocenter is uniformly drawn
-                from depth - delta_depth to depth + delta_depth (kilometers)
-                (Default: 20.0)
-            ztor (float): Fix the top of rupture to a specific depth (kilometers,
-                positive down). Negative values allow the TOR to float. (Default -1.)
-            seed (int): A seed for the random number generator (for testing only).
+            min_strike/max_strike (float/float): Limit the range of the strike
+                (in degrees) of realized faults. The default -180/180 allows
+                any strike.
+            min_dip/max_dip (float/float): Limit the range of the dip (in
+                degrees) of the realized faults. The default is to use the
+                "mech" of the origin to limit the range or, if none, use
+                the "ALL" mech which defaults to 35/90.
+            dx_min_frac/dx_max_frac (float/float): Limit the range of the
+                along-strike placement of the hypocenter to from
+                dx_min_frac * length to dx_max_frac * length. (default
+                0.0/1.0)
+            delta_depth (float): The depth range of the hypocenter is
+                uniformly drawn from depth - delta_depth to
+                depth + delta_depth (kilometers) (Default: 20.0)
+            ztor (float): Fix the top of rupture to a specific depth
+                (kilometers, positive down). Negative values allow the
+                TOR to float. (Default -1.)
+            seed (int): A seed for the random number generator (for testing
+                only).
         """
         self.origin = origin
         self.nsim = nsim
@@ -634,8 +719,8 @@ class RandomFiniteFault(object):
         """
         Initialize the random arrays
         """
-        (mscale, aspect, mindip_deg, maxdip_deg, szdepth, smech) = get_scaling_params(
-            self.origin
+        (mscale, aspect, mindip_deg, maxdip_deg, szdepth, smech) = (
+            get_scaling_params(self.origin)
         )
         _, _, _, _, area_mean, area_std = dimensions_from_magnitude(
             self.origin.mag, mscale, 1, 0, smech
@@ -649,7 +734,9 @@ class RandomFiniteFault(object):
             np.minimum(depth + delta_depth, szdepth),
             self.nsim,
         )
-        area = 10 ** self.rng.normal(np.log10(area_mean[0]), area_std, self.nsim)
+        area = 10 ** self.rng.normal(
+            np.log10(area_mean[0]), area_std, self.nsim
+        )
         width = np.sqrt(area / aspect)
         if min_dip is not None:
             mindip_deg = min_dip
@@ -718,7 +805,8 @@ class RandomFiniteFault(object):
         """
         if isim > self.nsim:
             raise IndexError(
-                f"RandomFiniteFault.retrieve_rupt {isim=} exceeds maximum {self.nsim}"
+                f"RandomFiniteFault.retrieve_rupt {isim=} exceeds "
+                "maximum {self.nsim}"
             )
         return self.rupts[isim]
 
@@ -825,13 +913,15 @@ def get_scaling_params(origin):
     return (mscale, aspect, mindip_deg, maxdip_deg, szdepth, smech)
 
 
-def dimensions_from_magnitude(M, rup_dim_model, neps, trunc, mech=Mechanism.ALL):
+def dimensions_from_magnitude(
+    mag, rup_dim_model, neps, trunc, mech=Mechanism.ALL
+):
     """
     Compute dimensions of rupture from magnitude for a specified
     magnitude scaling relation.
 
     Args:
-        M (float): Magnitude.
+        mag (float): Magnitude.
         rup_dim_model (MagScaling enum): Specifies the model for compputing the
             rupture dimensions from magnitude.
         neps (int): The number of steps to integrate from -trunc to +trunc.
@@ -865,32 +955,32 @@ def dimensions_from_magnitude(M, rup_dim_model, neps, trunc, mech=Mechanism.ALL)
         # Coppersmith.
         if mech is Mechanism.SS:
             sig_length = 0.15
-            length = 10 ** (-2.57 + 0.62 * M + sig_length * epsmid)
+            length = 10 ** (-2.57 + 0.62 * mag + sig_length * epsmid)
             sig_width = 0.14
-            width = 10 ** (-0.76 + 0.27 * M + sig_width * epsmid)
+            width = 10 ** (-0.76 + 0.27 * mag + sig_width * epsmid)
             sig_area = 0.22
-            area = 10 ** (-3.42 + 0.90 * M + sig_area * epsmid)
+            area = 10 ** (-3.42 + 0.90 * mag + sig_area * epsmid)
         elif mech is Mechanism.RS:
             sig_length = 0.16
-            length = 10 ** (-2.42 + 0.58 * M + sig_length * epsmid)
+            length = 10 ** (-2.42 + 0.58 * mag + sig_length * epsmid)
             sig_width = 0.15
-            width = 10 ** (-1.61 + 0.41 * M + sig_width * epsmid)
+            width = 10 ** (-1.61 + 0.41 * mag + sig_width * epsmid)
             sig_area = 0.26
-            area = 10 ** (-3.99 + 0.98 * M + sig_area * epsmid)
+            area = 10 ** (-3.99 + 0.98 * mag + sig_area * epsmid)
         elif mech is Mechanism.NM:
             sig_length = 0.17
-            length = 10 ** (-1.88 + 0.50 * M + sig_length * epsmid)
+            length = 10 ** (-1.88 + 0.50 * mag + sig_length * epsmid)
             sig_width = 0.12
-            width = 10 ** (-1.14 + 0.35 * M + sig_width * epsmid)
+            width = 10 ** (-1.14 + 0.35 * mag + sig_width * epsmid)
             sig_area = 0.22
-            area = 10 ** (-2.78 + 0.82 * M + sig_area * epsmid)
+            area = 10 ** (-2.78 + 0.82 * mag + sig_area * epsmid)
         elif mech is Mechanism.ALL:
             sig_length = 0.16
-            length = 10 ** (-2.44 + 0.59 * M + sig_length * epsmid)
+            length = 10 ** (-2.44 + 0.59 * mag + sig_length * epsmid)
             sig_width = 0.15
-            width = 10 ** (-1.01 + 0.32 * M + sig_width * epsmid)
+            width = 10 ** (-1.01 + 0.32 * mag + sig_width * epsmid)
             sig_area = 0.24
-            area = 10 ** (-3.49 + 0.91 * M + sig_area * epsmid)
+            area = 10 ** (-3.49 + 0.91 * mag + sig_area * epsmid)
         else:
             raise TypeError("Unsupported value of 'mech'")
     elif rup_dim_model is MagScaling.S14:
@@ -898,7 +988,7 @@ def dimensions_from_magnitude(M, rup_dim_model, neps, trunc, mech=Mechanism.ALL)
         #     - No length or width
         #     - No mechanism dependence
         sig_area = 0.3
-        area = 10 ** (M - 4.25 + sig_area * epsmid)
+        area = 10 ** (mag - 4.25 + sig_area * epsmid)
         length = None
         sig_length = None
         width = None
@@ -930,12 +1020,12 @@ def dimensions_from_magnitude(M, rup_dim_model, neps, trunc, mech=Mechanism.ALL)
         # So
         #     0.236*1   = 0.236 (pretty close to WC94)
         #     0.212*3/4 = 0.159 (seems a bit low...)
-        if M > 6.71:
+        if mag > 6.71:
             sig_area = 0.236
-            area = 10 ** (3 / 4 * (M - 3.07) + sig_area * epsmid)
+            area = 10 ** (3 / 4 * (mag - 3.07) + sig_area * epsmid)
         else:
             sig_area = 0.159
-            area = 10 ** ((M - 3.98) + sig_area * epsmid)
+            area = 10 ** ((mag - 3.98) + sig_area * epsmid)
         length = None
         sig_length = None
         width = None
@@ -943,19 +1033,19 @@ def dimensions_from_magnitude(M, rup_dim_model, neps, trunc, mech=Mechanism.ALL)
     elif rup_dim_model == MagScaling.Sea10_interface:
         # Strasser et al. (2010), coefficients for interface events
         sig_length = 0.18
-        length = 10 ** (-2.477 + 0.585 * M + sig_length * epsmid)
+        length = 10 ** (-2.477 + 0.585 * mag + sig_length * epsmid)
         sig_width = 0.173
-        width = 10 ** (-0.882 + 0.351 * M + sig_width * epsmid)
+        width = 10 ** (-0.882 + 0.351 * mag + sig_width * epsmid)
         sig_area = 0.304
-        area = 10 ** (-3.49 + 0.952 * M + sig_area * epsmid)
+        area = 10 ** (-3.49 + 0.952 * mag + sig_area * epsmid)
     elif rup_dim_model == MagScaling.Sea10_slab:
         # Strasser et al. (2010), coefficients for slab events
         sig_length = 0.146
-        length = 10 ** (-2.35 + 0.562 * M + sig_length * epsmid)
+        length = 10 ** (-2.35 + 0.562 * mag + sig_length * epsmid)
         sig_width = 0.067
-        width = 10 ** (-1.058 + 0.356 * M + sig_width * epsmid)
+        width = 10 ** (-1.058 + 0.356 * mag + sig_width * epsmid)
         sig_area = 0.184
-        area = 10 ** (-3.225 + 0.89 * M + sig_area * epsmid)
+        area = 10 ** (-3.225 + 0.89 * mag + sig_area * epsmid)
     else:
         raise TypeError("Unsupported value of 'rup_dim_model'")
     return length, sig_length, width, sig_width, area, sig_area
@@ -986,3 +1076,20 @@ def compute_epsilon(neps, trunc):
     if neps > 1:
         peps /= epsfac
     return epsmid, peps, d_eps
+
+
+# import concurrent.futures
+#        with concurrent.futures.ProcessPoolExecutor() as executor:
+#            futures = {
+#                executor.submit(call_gmpe, ix, gmpes, oqimt, cx, sd_types)
+#                for ix, oqimt in enumerate(myimts)
+#            }
+#            for fut in concurrent.futures.as_completed(futures):
+#                (ix, mean, stddevs) = fut.result()
+#
+# def call_gmpe(ix, gmpes, oqimt, cx, sd_types):
+#
+#    mean, stddevs = gmpes[oqimt].get_mean_and_stddevs(
+#        cx, cx, cx, [oqimt], sd_types
+#    )
+#    return (ix, mean, stddevs)
