@@ -35,7 +35,9 @@ def gmpe_gmas(gmpe, cx, imt, stddev_types):
     try:
         gmpe.compute(cx, imtl, mean, sig, tau, phi)
     except NotImplementedError:
-        mean, stddevs = gmpe.get_mean_and_stddevs(cx, cx, cx, imt, stddev_types)
+        mean, stddevs = gmpe.get_mean_and_stddevs(
+            cx, cx, cx, imt, stddev_types
+        )
         return mean, stddevs
     except Exception as exc:
         raise exc
