@@ -37,7 +37,10 @@ TABLES = OrderedDict(
                 )
             ),
         ),
-        ("imt", OrderedDict((("id", "integer primary key"), ("imt_type", "text")))),
+        (
+            "imt",
+            OrderedDict((("id", "integer primary key"), ("imt_type", "text"))),
+        ),
         (
             "amp",
             OrderedDict(
@@ -85,6 +88,9 @@ PSA_TO_SA = {
 
 
 def convert_units(imt_type, input_units, amplitude):
+    """
+    Convert amp units based on units strings.
+    """
     if re.search(r"PGA|SA", imt_type) is not None:
         # convert to ln(g)7
         if input_units in ["cm/s^2", "cm/s**2", "cm/s/s", "gals"]:
@@ -228,8 +234,8 @@ class StationList(object):
             filelist (sequence of str):
                 Sequence of ShakeMap XML and/or JSON input files to read.
             min_nresp (int):
-                The minimum number of DYFI observations required to form and valid
-                observation. Default is 3.
+                The minimum number of DYFI observations required to form and
+                valid observation. Default is 3.
             dbfile (str):
                 Path to a file into which to write the SQLite database.
                 The default is ':memory:' for an in-memory database.
@@ -252,8 +258,8 @@ class StationList(object):
             filelist:
                 A list of ShakeMap XML or JSON input files.
             min_nresp (int):
-                The minimum number of DYFI observations required to form and valid
-                observation.
+                The minimum number of DYFI observations required to form and
+                valid observation.
 
         Returns:
             nothing: Nothing.
@@ -264,6 +270,7 @@ class StationList(object):
             self._loadFromJSON(jsonfiles, min_nresp)
         if len(xmlfiles):
             self._loadFromXML(xmlfiles, min_nresp)
+        self._fixOrientations()
         return self
 
     def _loadFromXML(self, xmlfiles, min_nresp):
@@ -285,22 +292,29 @@ class StationList(object):
         stationdict = {}
         imtset = set()
         for xmlfile in xmlfiles:
-            stationdict, ims = self._filter_station(xmlfile, stationdict, min_nresp)
+            stationdict, ims = self._filter_station(
+                xmlfile, stationdict, min_nresp
+            )
             imtset |= ims
         # fill the database and create the object from it
         self._loadFromDict(stationdict, imtset)
-        self._fixOrientations()
         return
 
-    def _parse_shakemap_json(self, jfile, min_nresp, sta_set, imt_set, amp_set):
-        with open(jfile, "r") as jfp:
+    def _parse_shakemap_json(
+        self, jfile, min_nresp, sta_set, imt_set, amp_set
+    ):
+        with open(jfile, "r", encoding="utf-8") as jfp:
             stas = json.load(jfp)
 
         if "type" not in stas:
-            logging.warn(f"{jfile} appears to contain no stations, skipping")
+            logging.warning(
+                "%s appears to contain no stations, skipping", jfile
+            )
             return ([], [], [])
         if stas["type"] != "FeatureCollection":
-            logging.warn(f"{jfile} is not a ShakeMap JSON stationlist, skipping")
+            logging.warning(
+                "%s is not a ShakeMap JSON stationlist, skipping", jfile
+            )
             return ([], [], [])
 
         # if present, insert reference stuff into the database
@@ -353,7 +367,9 @@ class StationList(object):
 
             if not instrumented:
                 try:
-                    amplitude = float(feature["properties"].get("intensity", np.nan))
+                    amplitude = float(
+                        feature["properties"].get("intensity", np.nan)
+                    )
                 except (ValueError, TypeError):
                     amplitude = np.nan
                 try:
@@ -415,7 +431,9 @@ class StationList(object):
                         stddev = amp["sigma"]
                     else:
                         stddev = 0
-                    flag = amp["flag"]
+                    # We don't care about previous flagging, we'll do our own
+                    # flag = amp["flag"]
+                    flag = "0"
                     units = amp["units"]
                     if (
                         amplitude == "null"
@@ -453,10 +471,11 @@ class StationList(object):
             station_id = f"{network_code}.{station_code}"
             if station_id in sta_set:
                 continue
-            else:
-                sta_set.add(station_id)
+            sta_set.add(station_id)
             station_name = feature.properties.name
-            station_lon, station_lat, station_elev = feature.geometry.coordinates
+            station_lon, station_lat, station_elev = (
+                feature.geometry.coordinates
+            )
             vs30 = None
             station_rows.append(
                 (
@@ -478,7 +497,7 @@ class StationList(object):
                 for trace in stream.traces:
                     channel_code = trace.properties.channel_code
                     location_code = trace.properties.location_code
-                    if not len(location_code.strip()):
+                    if len(location_code.strip()) == 0:
                         location_code = "--"
                     is_channel = True
                     for pattern in NON_CHANNEL_PATTERNS:
@@ -491,8 +510,8 @@ class StationList(object):
                     if channel_code.lower().startswith("rot"):
                         orient = "H"
                     else:
-                        if channel_code[-1] in ORIENTATIONS:
-                            orient = ORIENTATIONS[channel_code[-1]]
+                        if channel_code[-1].upper() in ORIENTATIONS:
+                            orient = ORIENTATIONS[channel_code[-1].upper()]
                         else:
                             orient = None
                     orientation = _getOrientation(channel_code, orient)
@@ -526,17 +545,21 @@ class StationList(object):
                         else:
                             for idx, amplitude in enumerate(metric.values[0]):
                                 amplitude = convert_units(
-                                    imt_type, metric.properties.units, amplitude
+                                    imt_type,
+                                    metric.properties.units,
+                                    amplitude,
                                 )
                                 period = metric.dimensions.axis_values[1][idx]
                                 if period >= 1:
                                     imt_type = f"{metric.properties.name}({period:.1f})"
                                 else:
-                                    imt_type = f"{metric.properties.name}({period:g})"
+                                    imt_type = (
+                                        f"{metric.properties.name}({period:g})"
+                                    )
                                 imt_set.add(imt_type)
                                 amp_id = (
-                                    f"{network_code}.{station_code}.{imt_type}."
-                                    f"{location_code}.{channel_code}"
+                                    f"{network_code}.{station_code}.{imt_type}"
+                                    f".{location_code}.{channel_code}"
                                 )
                                 if amp_id in amp_set:
                                     continue
@@ -576,7 +599,7 @@ class StationList(object):
         #
         query = "SELECT id FROM station"
         self.cursor.execute(query)
-        sta_set = set([z[0] for z in self.cursor.fetchall()])
+        sta_set = {z[0] for z in self.cursor.fetchall()}
 
         orig_imt_set = self.getIMTtypes()
         imt_set = orig_imt_set.copy()
@@ -687,13 +710,13 @@ class StationList(object):
                 "SELECT a.amp, i.imt_type, a.original_channel, "
                 "a.flag, a.stddev, a.orientation, a.nresp "
                 "FROM amp a, imt i "
-                'WHERE a.station_id = "%s" '
-                "AND a.imt_id = i.id" % (str(sta[0]))
+                f"WHERE a.station_id = '{str(sta[0])}' "
+                "AND a.imt_id = i.id"
             )
             amp_rows = self.cursor.fetchall()
             if sta[8] == 0:
                 if len(amp_rows) != 1:
-                    logging.warn("Couldn't find intensity for MMI station.")
+                    logging.warning("Couldn't find intensity for MMI station.")
                     continue
                 feature["properties"]["intensity"] = amp_rows[0][0]
                 feature["properties"]["intensity_stddev"] = amp_rows[0][4]
@@ -790,7 +813,7 @@ class StationList(object):
         #
         query = "SELECT id FROM station"
         self.cursor.execute(query)
-        sta_set = set([z[0] for z in self.cursor.fetchall()])
+        sta_set = {z[0] for z in self.cursor.fetchall()}
 
         #
         # Insert any new stations into the station table
@@ -853,7 +876,9 @@ class StationList(object):
         self.cursor.execute(query)
         amp_rows = self.cursor.fetchall()
         # Create a unique identifier for each amp so we don't repeat any
-        amp_set = set([str(v[0]) + "." + str(v[1]) + "." + str(v[2]) for v in amp_rows])
+        amp_set = set(
+            [str(v[0]) + "." + str(v[1]) + "." + str(v[2]) for v in amp_rows]
+        )
 
         #
         # Insert the amps for each component
@@ -861,27 +886,36 @@ class StationList(object):
         amp_rows = []
         for sta_id, station_tpl in stationdict.items():
             station_attributes, comp_dict = station_tpl
-            instrumented = int(station_attributes["netid"].lower() not in CIIM_TUPLE)
+            instrumented = int(
+                station_attributes["netid"].lower() not in CIIM_TUPLE
+            )
             for original_channel, cdict in comp_dict.items():
                 pgm_dict = cdict["amps"]
-                if len(comp_dict) == 1 and original_channel == station_attributes.get(
+                if len(
+                    comp_dict
+                ) == 1 and original_channel == station_attributes.get(
                     "name", ""
                 ):
                     orientation = "H"
                 else:
                     orientation = cdict["attrs"].get("orientation", None)
-                    orientation = _getOrientation(original_channel, orientation)
+                    orientation = _getOrientation(
+                        original_channel, orientation
+                    )
                 for imt_type, imt_dict in pgm_dict.items():
                     if (instrumented == 0) and (imt_type != "MMI"):
                         continue
                     imtid = imt_hash[imt_type]
                     amp_id = (
-                        str(sta_id) + "." + str(imtid) + "." + str(original_channel)
+                        str(sta_id)
+                        + "."
+                        + str(imtid)
+                        + "."
+                        + str(original_channel)
                     )
                     if amp_id in amp_set:
                         continue
-                    else:
-                        amp_set.add(amp_id)
+                    amp_set.add(amp_id)
                     amp = imt_dict["value"]
                     units = imt_dict["units"]
                     stddev = imt_dict["stddev"]
@@ -952,10 +986,13 @@ class StationList(object):
             A set of IMT types
         """
         self.cursor.execute("SELECT imt_type FROM imt")
-        return set([z[0] for z in self.cursor.fetchall()])
+        return {z[0] for z in self.cursor.fetchall()}
 
     def getStationDictionary(
-        self, component="GREATER_OF_TWO_HORIZONTAL", instrumented=True, min_nresp=1
+        self,
+        component="GREATER_OF_TWO_HORIZONTAL",
+        instrumented=True,
+        min_nresp=1,
     ):
         """
         Return a dictionary of the instrumented or non-instrumented
@@ -988,7 +1025,8 @@ class StationList(object):
             component (str):
                 Defines the component of the data returned. The default is
                 "GREATER_OF_TWO_HORIZONTAL". Other choices might be "RotD50",
-                "RotD100", etc. Invalid components will simply not return any IMTs.
+                "RotD100", etc. Invalid components will simply not return any
+                IMTs.
             min_nresp (int):
                 The minimum number of DYFI responses required to make a valid
                 observation.
@@ -1009,7 +1047,7 @@ class StationList(object):
         columns = list(TABLES["station"].keys())
         dstr = ", ".join(columns)
         self.cursor.execute(
-            "SELECT %s FROM station where instrumented = %d" % (dstr, instrumented)
+            f"SELECT {dstr} FROM station where instrumented = {instrumented}"
         )
 
         station_rows = self.cursor.fetchall()
@@ -1040,13 +1078,14 @@ class StationList(object):
         #
         self.cursor.execute(
             "SELECT "
-            "a.amp, i.imt_type, a.station_id, a.stddev, a.nresp, a.original_channel "
+            "a.amp, i.imt_type, a.station_id, a.stddev, a.nresp, "
+            "a.original_channel "
             "FROM "
             "amp a, station s, imt i "
-            'WHERE a.flag = "0" '
+            "WHERE a.flag = '0' "
             "AND s.id = a.station_id "
             "AND a.imt_id = i.id "
-            'AND s.instrumented = ? AND a.orientation NOT IN ("Z", "U") '
+            "AND s.instrumented = ? AND a.orientation NOT IN ('Z', 'U') "
             "AND a.amp IS NOT NULL "
             "AND (a.nresp < 0 OR a.nresp >= ?)",
             (
@@ -1129,12 +1168,14 @@ class StationList(object):
             elif station_id == current_station_id:
                 channel_list.append(channel)
             if len(channel_list) == 3:
-                ochars = [x[-1] for x in channel_list]
+                ochars = [x[-1].upper() for x in channel_list]
                 if "1" in ochars and "2" in ochars and "Z" in ochars:
-                    hinds = [i for i, x in enumerate(ochars) if x == "1" or x == "2"]
+                    hinds = [
+                        i for i, x in enumerate(ochars) if x in ("1", "2")
+                    ]
                     sql = (
                         "UPDATE amp "
-                        'SET orientation = "H"'
+                        "SET orientation = 'H'"
                         "WHERE station_id = ? "
                         "AND original_channel IN (?, ?)"
                     )
@@ -1147,6 +1188,21 @@ class StationList(object):
                         ),
                     )
                     self.db.commit()
+                    hinds = [i for i, x in enumerate(ochars) if x == "Z"]
+                    sql = (
+                        "UPDATE amp "
+                        "SET orientation = 'Z' "
+                        "WHERE station_id = ? "
+                        "AND original_channel = ?"
+                    )
+                    self.cursor.execute(
+                        sql,
+                        (
+                            current_station_id,
+                            channel_list[hinds[0]],
+                        ),
+                    )
+                    self.db.commit()
                 # 3 channels, but not 1, 2, Z: bummer
                 channel_list = []
             # Channels weren't 3 so we can't fix; bummer
@@ -1190,13 +1246,14 @@ class StationList(object):
                 try:
                     value = float(pgm.attrib["value"])
                 except (ValueError, TypeError):
-                    logging.warn(
-                        "Unknown value in XML: %s for amp: %s"
-                        % (pgm.attrib["value"], pgm.tag)
+                    logging.warning(
+                        "Unknown value in XML: %s for amp: %s",
+                        pgm.attrib["value"],
+                        pgm.tag,
                     )
                     continue
             else:
-                logging.warn(f"No value for amp {pgm.tag}")
+                logging.warning("No value for amp %s", pgm.tag)
                 continue
             if "flag" in pgm.attrib and pgm.attrib["flag"] != "":
                 flag = pgm.attrib["flag"]
@@ -1233,8 +1290,8 @@ class StationList(object):
                 Path to ShakeMap XML input file (or file-like object)
                 containing station data.
             stationdict (dict):
-                the dictionary of stations that the stations in this file should
-                be added to.
+                the dictionary of stations that the stations in this file
+                should be added to.
             min_nresp (int):
                 The minimum number of responses that a DYFI observation needs
                 to be included in the processing.
@@ -1263,13 +1320,15 @@ class StationList(object):
                 shortref = ""
                 description = ""
                 query = (
-                    f"INSERT INTO reference "
+                    "INSERT INTO reference "
                     "(shortref, longref, description) VALUES "
-                    f'("{shortref}","{longref}","{description}")'
+                    f"('{shortref}','{longref}','{description}')"
                 )
                 self.cursor.execute(query)
                 self.db.commit()
-                query = f"SELECT id FROM reference WHERE longref is " f'("{longref}")'
+                query = (
+                    f"SELECT id FROM reference WHERE longref is ('{longref}')"
+                )
                 self.cursor.execute(query)
                 row = self.cursor.fetchall()
                 refid = row[0][0]
@@ -1284,7 +1343,7 @@ class StationList(object):
                 attributes = station.attrib.copy()
                 if "netid" in attributes:
                     netid = attributes["netid"]
-                    if not len(netid.strip()):
+                    if len(netid.strip()) == 0:
                         netid = "unknown"
                 else:
                     netid = "unknown"
@@ -1293,7 +1352,9 @@ class StationList(object):
                 instrumented = int(netid.lower() not in CIIM_TUPLE)
 
                 if "code" not in attributes:
-                    logging.warn("Station does not have station code: skipping")
+                    logging.warning(
+                        "Station does not have station code: skipping"
+                    )
                     continue
                 code = attributes["code"]
                 if code.startswith(netid + "."):
@@ -1309,8 +1370,9 @@ class StationList(object):
                     compdict = {}
                 for comp in station:
                     if "name" not in comp.attrib:
-                        logging.warn(
-                            f"Unnamed component for station {sta_id}; skipping"
+                        logging.warning(
+                            "Unnamed component for station %s; skipping",
+                            sta_id,
                         )
                         continue
                     compname = comp.attrib["name"]
@@ -1364,10 +1426,10 @@ class StationList(object):
         """
         Build the database tables.
         """
-        for table in TABLES.keys():
+        for table, columns in TABLES.items():
             sql = f"CREATE TABLE {table} ("
             nuggets = []
-            for column, ctype in TABLES[table].items():
+            for column, ctype in columns.items():
                 nuggets.append(f"{column} {ctype}")
             sql += ",".join(nuggets) + ")"
             self.cursor.execute(sql)
@@ -1427,17 +1489,17 @@ def _getOrientation(orig_channel, orient):
     if orient == "H":
         return "H"
 
-    if not len(orig_channel.strip()):
+    if len(orig_channel.strip()) == 0:
         orientation = "U"  # default when we don't know anything about channel
         return orientation
 
     if orig_channel == "mmi" or orig_channel == "DERIVED":
         orientation = "H"  # mmi is arbitrarily horizontal
-    elif orig_channel[-1] in ("N", "E", "Z"):
-        orientation = orig_channel[-1]
+    elif orig_channel[-1].upper() in ("N", "E", "Z"):
+        orientation = orig_channel[-1].upper()
     elif orig_channel == "UNK":  # Channel is "UNK"; assume horizontal
         orientation = "H"
-    elif orig_channel == "H1" or orig_channel == "H2":
+    elif orig_channel.upper() == "H1" or orig_channel.upper() == "H2":
         orientation = "H"
     elif orig_channel[-1].isdigit():
         if orient == "h":
@@ -1468,9 +1530,13 @@ def _getOrientationSet(chan_names):
         'E', 'Z', 'H' (for horizontal), or 'U' (for unknown).
     """
     if len(chan_names) == 3:
-        term_chars = [chan_names[0][-1], chan_names[1][-1], chan_names[2][-1]]
+        term_chars = [
+            chan_names[0][-1].upper(),
+            chan_names[1][-1].upper(),
+            chan_names[2][-1].upper(),
+        ]
         if "1" in term_chars and "Z" in term_chars:
-            orientations = [(lambda x: "Z" if x == "Z" else "H")(x) for x in term_chars]
+            orientations = [("Z" if x == "Z" else "H") for x in term_chars]
             return orientations
     orientations = []
     for name in chan_names:
